;;; -*- Emacs-Lisp -*-
;; mew-fake-cdp -- Un-official attachment filename support.
;;

;; Create: 98/3/13
;; Author: Shun-ichi GOTO <gotoh@taiyo.co.jp>
;;
;; $Revision: 1.13 $

;; *********************************************************
;; ******* NOTE: THESE ARE OUT OF RFC SPECIFICATION. *******
;; *******       USE THIS WITH YOUR OWN RISK.        *******
;; *********************************************************

;; To use mew-fake-cdp.el, put followings in your .emacs
;;
;; (eval-after-load "mew" '(load "mew-fake-cdp"))
;;
;; You can enable or disable this feature with interactive commands
;; mew-fake-cdp-activate or mew-fake-cdp-deactivate.
;;

;; *** NOTE FOR OLD VERSION USERS ***
;; mew-fake-cdp.el uses ADVICE package from rev.1.11 that activats
;; faking feature automaticaly when load mew-fake-cdp.el.
;; So function mew-fake-cdp-setup is obsoleted.
;; (But it remains as empty function for compatibility.)
;; 

(require 'mew-decode)
(require 'mew-bq)
(require 'advice)

(defvar mew-fake-cdp-use-cd nil
  "*Use Content-Description: as filename")

(defvar mew-fake-cdp-use-ct t
  "*Use Content-Type: name=xxx as filename")

(defvar mew-fake-cdp-remove-space nil
  "*Remove space/TAB characters in filename string")

(defvar mew-fake-cdp-allow-non-ascii-coding t
  "*Decode MIME parameters to support raw-JIS filename.")


;; internal use variable
(defvar mew-fake-cdp-new-syntax-p
  (if (not (string-match "[a-zA-Z ]+\\([1-9]+\\).\\([0-9]+\\)\\(b[0-9]+\\)?"
			 mew-version))
      nil				; unknown => old-type
    (or (<= (string-to-int (mew-match 1 mew-version)) 1)
	(<= (string-to-int (mew-match 2 mew-version)) 94)
	(<= (string-to-int (mew-match 3 mew-version)) 6))))


;; functions

(defun mew-fake-cdp-setup ()
  "OBSOLETED: Setup to use mew-fake-cdp package"
  ;; null function for compatibility
  )

(defun mew-fake-cdp-find-cs-string (string)
  (let ((buf (get-buffer-create "*mew-find-cs-string*"))
	cs)
    (unwind-protect
	(save-excursion
	  (set-buffer buf)
	  (erase-buffer)
	  (insert string)
	  (setq cs (mew-find-cs-region (point-min) (point-max))))
      (kill-buffer buf))
    cs))

(defun mew-fake-cdp-decode-string-list (item)
  "Decode ITEM which is string or list of string.
When list, decode only if string element in ITEM is string for each.
Result is returned as same structure to given ITEM.
"
  (cond
   ((stringp item)			; case of siple string (not list)
    (if (equal (mew-fake-cdp-find-cs-string item) (list mew-lc-ascii))
	(mew-cs-decode-string item mew-cs-autoconv)
      item))
   ((listp item)			; list of string
    (mapcar 
     '(lambda (x) (cond
		   ((and (stringp x)
			 (equal (mew-fake-cdp-find-cs-string x)
				(list mew-lc-ascii)))
		    (mew-cs-decode-string x mew-cs-autoconv))
		   ((listp x) (mew-fake-cdp-decode-string-list x))
		   (t x)))
     item))
   (t item)))				; others (nothing to do)

(if (fboundp 'mew-syntax-get-member)
    (defalias 'mew-fake-cdp-syntax-get-member 'mew-syntax-get-member)
  ;; sanity-check in mew-syntax-get-param is not good for raw-JIS decoding.
  (defun mew-fake-cdp-syntax-get-member (ctl member)
    "Same to mew-syntax-get-member except no sanity-check."
    (car (cdr (assoc member ctl)))))

(defadvice mew-param-analyze (before fake-cdp (param) activate)
  "Allow multi-lined value in parameter (for mew-1.94b7 or later)."
  (if (not mew-fake-cdp-new-syntax-p)
      ()
    (while (string-match "\n " param)
      (setq param (concat (substring param 0 (match-beginning 0))
			  (substring param (match-end 0)))))))


(defadvice mew-decode-mime-header (around fake-cdp (&optional dct) activate)
  "Altanative function of mew-mew-decode-mime-header 
to support un-official attachment filename extraction.

This package provides:
 1. Make CDP: entry if CDP: is not specified and
    name= parameter of CT: is specified.
 2. Accept B/Q-Decoded filename. (like MIMEed Subject: header)
 3. Accept raw JIS filename string. (assume iso-2022-jp-2)
 4. Use Content-Description: as filename (optional)

NOTE: THESE ARE OUT OF RFC SPECIFICATION.
      USE THIS WITH YOUR OWN RISK.
"
  (let* ((syntax ad-do-it)
	 (ctl (mew-syntax-get-ct syntax))
	 (cdpl (mew-syntax-get-cdp syntax))
	 (cd (mew-syntax-get-cd syntax))
	 filename
	 )
    (condition-case nil
	(progn
	  ;; deocde for raw-JIS filename support.
	  (if mew-fake-cdp-allow-non-ascii-coding
	      (progn
		(setq ctl  (mew-fake-cdp-decode-string-list ctl)
		      cdpl (mew-fake-cdp-decode-string-list cdpl)
		      cd   (mew-fake-cdp-decode-string-list cd))
		(mew-syntax-set-cd syntax cd)))
	  ;; 1. use CDP: filename=
	  ;; 2. use CT: name=
	  ;; 3. use CD: (optional)
	  (setq filename (or (and cdpl
				  (mew-fake-cdp-syntax-get-member cdpl "filename"))
			     (and mew-fake-cdp-use-ct ctl 
				  (mew-fake-cdp-syntax-get-member ctl "name"))
			     (and mew-fake-cdp-use-cd
				  cd)))
	  (if (null filename)
	      ()			;nothing to do
	    ;; MIME decode
	    (while (string-match mew-header-decode-regex filename)
	      (setq filename (concat (substring filename 0 (match-beginning 0))
				     (save-match-data
				       (mew-header-decode 
					(mew-match 1 filename)
					(mew-match 2 filename)
					(mew-match 3 filename)))
				     (substring filename (match-end 0)))))
	    ;; remove spaces
	    (if  mew-fake-cdp-remove-space
		(save-match-data
		  (while (string-match "[ \t]+" filename)
		    (setq filename (concat (substring filename
						      0 (match-beginning 0))
					   (substring filename
						      (match-end 0)))))))
	    ;; Fake CDP: entry
	    (if mew-fake-cdp-new-syntax-p ; new syntax format support, XXX
		(setq param (list "filename" filename))
	      (setq param (format "filename=%s" filename)))
	    (if (null cdpl)
		;; newly make
		(setq cdpl (list "attachment" param))
	      (if (mew-fake-cdp-syntax-get-member cdpl "filename")
		  ;; replace
		  (if mew-fake-cdp-new-syntax-p
		      (setq cdpl (append (delete (assoc "filename" cdpl) cdpl)
					 (list param)))
		    (setq cdpl (mapcar
				(lambda (x)
				  (let ((case-fold-search t))
				    (if (string-match "^filename=" x)
					(concat "filename=" filename)
				      x)))
				cdpl)))
		;; add
		(setq cdpl (append cdpl (list param)))
		))
	    (mew-syntax-set-cdp syntax cdpl)
	    )
	  ;; return
	  syntax)
      (error (message "WARNING! Some error occured in mew-fake-cdp.el")))
      ))


;; Commands to enable/disable mew-fake-cdp feature.

(defun mew-fake-cdp-activate ()
  (interactive)
  (ad-activate 'mew-decode-mime-header)
  (ad-activate 'mew-param-analyze)
  (message "mew-fake-cdp is ACTIVATED"))


(defun mew-fake-cdp-deactivate ()
  (interactive)
  (ad-deactivate 'mew-decode-mime-header)
  (ad-deactivate 'mew-param-analyze)
  (message "mew-fake-cdp is DEACTIVATED (original behaviour)"))


(provide 'mew-fake-cdp)

;;;
;;; end of mew-fake-cdp.el
;;;
